﻿using System.Collections.Generic;
using System.Linq;
using Neo4RestNet.Core;

namespace Neo4RestNet.ObjectMapping
{
  public class PropertyMap : IPropertyMap
  {
    private readonly IReflector _reflector;

    public PropertyMap(IReflector reflector)
    {
      _reflector = reflector;
    }

    private IEnumerable<Property> GetProperties(object o)
    {
      // Convention is to not map anything starting with _
      return _reflector.GetProperties(o.GetType()).Where(p => !p.Name.StartsWith("_"));
    }

    public object ContainerToObject(IPropertyContainer from, object to)
    {
      // Copy property values from container to object
      foreach (var property in GetIntersection(to, from))
      {
        property.Set(to, from.GetProperty<object>(property.Name));
      }
      return to;
    }

    public IPropertyContainer ObjectToContainer(object from, IPropertyContainer to)
    {      
      // Copy property values from object to container
      foreach (var property in GetProperties(from))
      {
        to.SetProperty(property.Name, property.Get(from));
      }
      return to;
    }

    public T Container2Object<T>(IPropertyContainer from, T to)
    {
      return (T) ContainerToObject(from, to);
    }

    private IEnumerable<Property> GetIntersection(object obj, IPropertyContainer container)
    {
      // Return the set of properties that exists both in container and on object
      return GetProperties(obj).Where(p => container.HasProperty(p.Name));
    }
  }
}
