﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Neo4RestNet.Core;

namespace Neo4RestNet.ObjectMapping.Reflection
{
  public class EmitReflector : IReflector
  {
    // Keeps a list of all types that has been reflected upon this is important since
    // we are dynamically emitting code for fast get/set operations
    private static readonly IDictionary<string, IEnumerable<Property>> _reflectedTypes
      = new Dictionary<string, IEnumerable<Property>>();

    private static readonly object _lock = new object();

    public IEnumerable<Property> GetProperties(Type type)
    {
      string typeName = type.ToString();

      if (_reflectedTypes.ContainsKey(typeName))
      {
        return _reflectedTypes[typeName];
      }

      lock (_lock)
      {
        // Check again, some other thread might have done the work for us while
        // we were hanging on the lock
        if (_reflectedTypes.ContainsKey(typeName))
        {
          return _reflectedTypes[typeName];
        }

        _reflectedTypes[typeName] = InitializeProperties(type);
      }

      return _reflectedTypes[typeName];
    }

    public EmptyConstructor GetEmptyConstructor(Type type)
    {
      throw new NotImplementedException();
    }

    public EmptyConstructor GetEmptyConstructor(string typeName)
    {
      throw new NotImplementedException();
    }

    private static IEnumerable<Property> InitializeProperties(Type type)
    {
      return GetPropertyInfo(type).Select(p => new Property(EmitHelper.CreatePropertyGetterHandler(p), EmitHelper.CreatePropertySetterHandler(p), p.Name));
    }

    private static IEnumerable<PropertyInfo> GetPropertyInfo(Type type)
    {
      // Only include properties whose type is supported by the container
      return 
        type.GetProperties(BindingFlags.Public | BindingFlags.Instance | BindingFlags.SetProperty | BindingFlags.GetProperty)
          .Where(p => PropertyContainer.IsTypeSupported(p.PropertyType));
    }
  }
}
