namespace Neo4RestNet.Core
{
  public interface IGraphDatabaseService
  {
    /// <summary>
    /// Adds a new node to the database
    /// </summary>
    /// <remarks>
    /// This isn't really true to the java api, an empty node is always created and properties are set one at the time. 
    /// For a rest interface this might get a bit chatty but passing in json isn't that nice in the long run. Some form
    /// of templating might be used, like a dictionary or another node..
    /// </remarks>
    /// <returns>The created node</returns>
    INode CreateNode(string data);

    INode CreateNode(IPropertyContainer propertyContainer);

    /// <summary>
    /// Adds a new node to the database
    /// </summary>
    /// <returns></returns>
    INode CreateNode();

    /// <summary>
    /// Returns the graphs reference node. The reference node is usually used as a starting point for the application
    /// </summary>
    /// <returns></returns>
    INode GetReferenceNode();

    /// <summary>
    /// Gets a single node by its native database identity
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    INode GetNode(long id);

    /// <summary>
    /// Gets a single node by its url
    /// </summary>
    /// <param name="self"></param>
    /// <returns></returns>
    INode GetNode(string self);
  }
}