﻿using System.Collections.Generic;

namespace Neo4RestNet.Core
{
  public interface IIndexService
  {
    /// <summary>
    /// Creates an index for the specified key / value and adds the node to it
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="node"></param>
    /// <param name="key"></param>
    /// <param name="value"></param>
    void AddIndex<T>(INode node, string key, T value);

    /// <summary>
    /// Removes the node from the index
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="node"></param>
    /// <param name="key"></param>
    /// <param name="value"></param>
    void RemoveIndex<T>(INode node, string key, T value);

    /// <summary>
    /// Returns all nodes in the specified index
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="key"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    IEnumerable<INode> GetNodes<T>(string key, T value);

    /// <summary>
    /// Returns a single node with specified key / value
    /// Throws an GraphDbException if more than one node is in the specified index
    /// Returns null if no node matches
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="key"></param>
    /// <param name="value"></param>
    /// <returns></returns>
    INode GetSingleNode<T>(string key, T value);
  }
}