﻿using System.Collections.Generic;

namespace Neo4RestNet.Core
{
  /// <summary>
  /// T should only be value types, array of value types, strings or array of strings
  /// </summary>
  public interface IPropertyContainer
  {
    /// <summary>
    /// Throws NotFoundException if key is not present
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="key"></param>
    /// <returns></returns>
    T GetProperty<T>(string key);

    /// <summary>
    /// Returns specified default value if key is not present
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="key"></param>
    /// <param name="defaultValue"></param>
    /// <returns></returns>
    T GetProperty<T>(string key, T defaultValue);

    /// <summary>
    /// Returns all keys
    /// </summary>
    /// <returns></returns>
    IEnumerable<string> GetPropertyKeys();

    /// <summary>
    /// Returns true if property exists, otherwise false
    /// </summary>
    /// <param name="key"></param>
    /// <returns></returns>
    bool HasProperty(string key);

    /// <summary>
    /// Removes the property
    /// </summary>
    /// <param name="key"></param>
    /// <returns></returns>
    object RemoveProperty(string key);

    /// <summary>
    /// Sets the property (!)
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="key"></param>
    /// <param name="value"></param>
    void SetProperty<T>(string key, T value);
  }
}