﻿using System;
using System.Collections.Generic;

namespace Neo4RestNet.Core
{
  public class PropertyContainer : IPropertyContainer
  {
    private static readonly HashSet<string> _handledTypes
      = new HashSet<string>
          {
            typeof(Int16).ToString(),
            typeof(Int32).ToString(),
            typeof(Int64).ToString(),
            typeof(byte).ToString(),
            typeof(decimal).ToString(),
            typeof(char).ToString(),
            typeof(float).ToString(),
            typeof(double).ToString(),
            typeof(string).ToString(),
            typeof(bool).ToString(),
            //typeof(int[]).ToString(),
            //typeof(byte[]).ToString(),
            //typeof(decimal[]).ToString(),
            //typeof(char[]).ToString(),
            //typeof(float[]).ToString(),
            //typeof(string[]).ToString(),
          };

    private readonly IDictionary<string, object> _container;
 
    public PropertyContainer()
    {
      _container = new Dictionary<string, object>();
    }

    public PropertyContainer(IDictionary<string, object> container)
    {
      _container = container;
    }

    public static bool IsTypeSupported(Type type)
    {
      return _handledTypes.Contains(type.ToString());
    }

    private static object Unbox<T>(object o)
    {
      return Convert.ChangeType(o, typeof (T));
    }

    public T GetProperty<T>(string key)
    {
      if (!_container.Keys.Contains(key)) throw new NotFoundException(string.Format("Failed to retrieve property {0}", key), null);
      return (T)Unbox<T>(_container[key]);
    }

    public T GetProperty<T>(string key, T defaultValue)
    {
      if (!_container.Keys.Contains(key)) return defaultValue;
      return (T)Unbox<T>(_container[key]);
    }

    public IEnumerable<string> GetPropertyKeys()
    {
      return _container.Keys;
    }

    public bool HasProperty(string key)
    {
      return _container.Keys.Contains(key);
    }

    public object RemoveProperty(string key)
    {
      // Return old value
      var oldValue = _container.Keys.Contains(key) ? _container[key] : null;
      // Remove from container
      _container.Remove(key);

      return oldValue;
    }

    public void SetProperty<T>(string key, T value)
    {
      if (!_handledTypes.Contains(value.GetType().ToString())) throw new ArgumentException(string.Format("Type {0} is not supported in propertycontainer", value.GetType()));
      // Set property in container
      _container[key] = value;
    }
  }
}