using System;
using System.Collections.Generic;

namespace Neo4RestNet.ExpressionParser
{
  public class AtomConverter
  {
    private delegate string ConvertConstant(object o);

    private static readonly IDictionary<string, ConvertConstant> _typeConverters
      = new Dictionary<string, ConvertConstant>
          {
            // Bool
            {typeof(bool).ToString(), 
              // Conversion to constant
              o => ((bool)o) ? "true" : "false"},

            // Int32
            {typeof(Int32).ToString(), 
              // Conversion to constant
              o => o.ToString()},

            // String
            {typeof(String).ToString(), 
              // Conversion to constant
              o => "'" + o.ToString() + "'"},
          };

    public string Convert(object o)
    {
      if (!IsAtom(o)) throw new Exception(string.Format("Unable to convert {0} to an atom", o.GetType()));
      return _typeConverters[o.GetType().ToString()](o);        
    } 

    public bool IsAtom(object o)
    {
      return _typeConverters.Keys.Contains(o.GetType().ToString());
    }
  }
}