using System;
using System.Linq.Expressions;
using System.Reflection;

namespace Neo4RestNet.ExpressionParser
{
  // Handles access to constants and instances of objects outside of the lambda
  public class ExpressionEvaluator
  {
    private readonly AtomConverter _atomConverter;

    public ExpressionEvaluator(AtomConverter atomConverter)
    {
      _atomConverter = atomConverter;
    }

    // Evaluate the member access, if some kind of atom is found, return it
    // Otherwise throw!
    public string EvaluateMemberAccess(MemberExpression memberExpression)
    {
      var innerExpression = memberExpression.Expression;
      switch (innerExpression.NodeType)
      {
        case ExpressionType.Constant:
          // Accessing a constant on the object
          return EvaluateConstant(memberExpression.Member, (ConstantExpression)innerExpression);
        default:
          throw new Exception("Unable to evaluate access to member located outside of the lambda");
      }
    }

    public string EvaluateMethodCall(MethodCallExpression methodCallExpression)
    {
      var resultFromMethodCall = DoEvaluateMethodCall(methodCallExpression);
      return _atomConverter.Convert(resultFromMethodCall);
    }

    private static object DoEvaluateMethodCall(MethodCallExpression methodCallExpression)
    {
      var method = methodCallExpression.Method;
      var methodInfo = methodCallExpression.Method.ReflectedType;
      return  methodInfo.InvokeMember(method.Name, BindingFlags.InvokeMethod, null, methodCallExpression.Object, new object[0]);      
    }

    private string EvaluateConstant(MemberInfo memberInfo, ConstantExpression constantExpression)
    {
      // Retrieve the constant value from the instance (constantExpression.Value)
      var value =  
        memberInfo.ReflectedType.InvokeMember(memberInfo.Name, BindingFlags.GetField, null, constantExpression.Value, new object[0]);
      return _atomConverter.Convert(value);
    }
  }
}