﻿using System.Collections.Generic;
using Neo4RestNet.Core;
using Newtonsoft.Json.Linq;

namespace Neo4RestNet.Rest.Parsers
{
  /// <summary>
  /// Parses a node json representation
  /// The json is on format: { 'self': '', 'data': {'property':'value', 'property':'value'}
  /// </summary>
  internal class NodeParser
  {
    private readonly PropertiesParser _propertiesParser;

    internal NodeParser()
    {
      _propertiesParser = new PropertiesParser();
    }

    /// <summary>
    /// Parses json string into a node
    /// </summary>
    /// <param name="nodeJson"></param>
    /// <returns></returns>
    internal INode Parse(string nodeJson)
    {
      return Parse(JObject.Parse(nodeJson));
    }

    /// <summary>
    /// Parses json token into a node
    /// </summary>
    /// <param name="node"></param>
    /// <returns></returns>
    internal INode Parse(IDictionary<string, JToken> node)
    {
      var self = node["self"].Value<string>();
      var data = node["data"];

      // data is the set of properties on the node
      var properties = _propertiesParser.Parse((JObject)data);
      return new RestNode(self, properties);
    }
  }
}
