﻿using System.Collections.Generic;
using Neo4RestNet.Core;
using Newtonsoft.Json.Linq;

namespace Neo4RestNet.Rest.Parsers
{
  public class RelationshipParser
  {
    private readonly PropertiesParser _propertiesParser;

    public RelationshipParser()
    {
      _propertiesParser = new PropertiesParser();
    }

    public IEnumerable<IRelationship> ParseArray(string relationshipArrayJson)
    {
      var relationships = JArray.Parse(relationshipArrayJson);

      foreach (var relationship in relationships)
      {
        yield return Parse((JObject) relationship);
      }
    }

    internal IRelationship Parse(string relationshipJson)
    {
      var relationship = JObject.Parse(relationshipJson);
      return Parse(relationship);
    }

    private IRelationship Parse(IDictionary<string, JToken> relationship)
    {
      var self = relationship["self"].Value<string>();
      var start = relationship["start"].Value<string>();
      var end = relationship["end"].Value<string>();
      var name = relationship["type"].Value<string>();
      var data = relationship["data"];

      // data is the set of properties on the node
      var properties = _propertiesParser.Parse((JObject)data);
      return new RestRelationship(self, new RestNode(start), new RestNode(end), name, properties);
    }
  }
}