using System.Net;
using Neo4RestNet.Core;
using Neo4RestNet.Rest.Encoders;
using Neo4RestNet.Rest.Parsers;

namespace Neo4RestNet.Rest
{
  public class RestGraphDatabaseService : IGraphDatabaseService
  {
    private readonly string _databaseUrl;
    private readonly NodeParser _nodeParser;

    /// <summary>
    /// Initializes a new database
    /// </summary>
    /// <param name="databaseUrl">For example http://localhost:9999/</param>
    public RestGraphDatabaseService(string databaseUrl)
    {
      // Remove trailing /
      _databaseUrl = databaseUrl.TrimEnd('/');
      _nodeParser = new NodeParser();
    }

    private static RestServer Server { get { return RestServer.Instance(); } } 

    public INode CreateNode(string data)
    {
      string createdNodeJson;

      // Create the node, the server returns the created node
      try
      {
        createdNodeJson = Server.Post(_databaseUrl + "/node", data);
      }
      catch (WebException e)
      {        
        throw new GraphDbException("Failed to create node", e);
      }

      return _nodeParser.Parse(createdNodeJson);
    }

    public INode CreateNode(IPropertyContainer propertyContainer)
    {
      return CreateNode(new PropertyContainerEncoder().Encode(propertyContainer));
    }

    public INode CreateNode()
    {
      return CreateNode("{}");
    }

    public INode GetReferenceNode()
    {
      // This one isn't right, we need to query for the ref node
      return new RestNode(_databaseUrl, 0);
    }

    public INode GetNode(long id)
    {
      return new RestNode(_databaseUrl, id);
    }

    public INode GetNode(string self)
    {
      return new RestNode(self);
    }
  }
}