﻿using System;
using System.Collections.Generic;
using System.Linq;
using Neo4RestNet.Core;
using Neo4RestNet.Rest.Parsers;

namespace Neo4RestNet.Rest
{
  public class RestIndexService : IIndexService 
  {
    private readonly string _databaseUrl;
    private readonly NodeArrayParser _nodeArrayParser;

    /// <summary>
    /// Initializes a new indexer
    /// </summary>
    /// <param name="databaseUrl">For example http://localhost:9999/</param>
    public RestIndexService(string databaseUrl)
    {
      // Remove trailing /
      _databaseUrl = databaseUrl.TrimEnd('/');
      _nodeArrayParser = new NodeArrayParser();
    }

    private static RestServer Server { get { return RestServer.Instance(); } } 

    public void AddIndex<T>(INode node, string key, T value)
    {
      Server.Post(_databaseUrl + "/index/node/" + key + "/" + value, node.Self);
    }

    public void RemoveIndex<T>(INode node, string key, T value)
    {
      Server.Delete(_databaseUrl + "/index/node/" + key + "/" + value + "/" + node.Id);
    }

    public IEnumerable<INode> GetNodes<T>(string key, T value)
    {
      var response = Server.Get(_databaseUrl + "/index/node/" + key + "/" + value);
      return _nodeArrayParser.Parse(response);
    }

    public INode GetSingleNode<T>(string key, T value)
    {
      try
      {
        return GetNodes(key, value).SingleOrDefault();
      }
      catch (InvalidOperationException e)
      {
        throw new GraphDbException(string.Format("More than one node matches index {0}={1}", key, value), e);
      }
    }
  }
}
