﻿using System.Collections.Generic;
using System.Net;
using Neo4RestNet.Core;

namespace Neo4RestNet.Rest
{
  public class RestRelationship : IRelationship
  {
    private readonly RestNode _startNode;
    private readonly RestNode _endNode;
    private readonly string _name;
    private readonly PropertyContainer _propertyContainer;

    public RestRelationship(string self, RestNode startNode, RestNode endNode, string name, IDictionary<string, object> properties)
    {
      Self = self;
      _startNode = startNode;
      _endNode = endNode;
      _name = name;
      InitializeFromSelf(self);
      _propertyContainer = new PropertyContainer(properties);
    }

    private void InitializeFromSelf(string self)
    {
      // Make sure there is no trailing /
      Self = self.TrimEnd('/');
    }

    private static RestServer Server { get { return RestServer.Instance(); } }

    private IPropertyContainer PropertyContainer
    {
      get { return _propertyContainer;  }
    }

    public T GetProperty<T>(string key)
    {
      return PropertyContainer.GetProperty<T>(key);      
    }

    public T GetProperty<T>(string key, T defaultValue)
    {
      return PropertyContainer.GetProperty(key, defaultValue);
    }

    public IEnumerable<string> GetPropertyKeys()
    {
      return PropertyContainer.GetPropertyKeys();
    }

    public bool HasProperty(string key)
    {
      return PropertyContainer.HasProperty(key);
    }

    public object RemoveProperty(string key)
    {
      // Old value might have to be read before deleting it
      var oldValue = PropertyContainer.RemoveProperty(key);
      Server.Delete(Self + "/properties/" + key);
      return oldValue;
    }

    public void SetProperty<T>(string key, T value)
    {
      Server.Put(Self + "/properties/" + key, value.ToString());
      PropertyContainer.SetProperty(key, value);
    }

    public void Delete()
    {
      try
      {
        Server.Delete(Self);
      }
      catch (WebException e)
      {
        throw new GraphDbException(string.Format("Failed to delete relationship {0}", Self), e);
      }
    }

    public INode EndNode
    {
      get { return _endNode; }
    }

    public INode StartNode
    {
      get { return _startNode; }
    }

    public INode GetOtherNode(INode node)
    {
      if (node.Id == _startNode.Id) return _endNode;
      if (node.Id == _endNode.Id) return _startNode;

      throw new GraphDbException("No other node found");
    }

    public INode[] Nodes
    {
      get { return new INode[] { _startNode, _endNode }; }
    }

    public string Name
    {
      get { return _name; }
    }

    public string Self { get; private set; }
  }
}
