using System;
using System.Net;
using System.Text;

namespace Neo4RestNet.Rest
{
  public class RestServer
  {
    private static readonly RestServer _restServer = new RestServer();

    private RestServer()
    {      
    }

    public static RestServer Instance()
    {
      return _restServer;
    }

    private static WebResponse SendRequest(string url, string method, string body)
    {
      var request = WebRequest.Create(url);
      request.Method = method;
      request.ContentType = "application/json";
      if (!string.IsNullOrEmpty(body))
      {
        var bytes = Encoding.UTF8.GetBytes(body);
        using (var stream = request.GetRequestStream())
        {
          stream.Write(bytes, 0, bytes.Length);                  
        }
      }
      return request.GetResponse();
    }

    private static string HandleResponse(WebResponse response)
    {
      using (response)
      using (var stream = response.GetResponseStream())
      {
        if (response.ContentLength <= 0) return string.Empty;
        if (response.ContentLength > Int32.MaxValue) throw new Exception("Only supports responses of Int32.MaxValue so far..");

        var length = (int)response.ContentLength;
        var buffer = new byte[length];
        stream.Read(buffer, 0, length);
        return Encoding.UTF8.GetString(buffer);
      }      
    }

    /// <summary>
    /// Http GET
    /// </summary>
    /// <param name="url">What to get</param>
    /// <returns>Response body from server</returns>
    public string Get(string url)
    {
      return HandleResponse(SendRequest(url, "GET", null));
    }

    /// <summary>
    /// Http POST
    /// </summary>
    /// <param name="url">Where to post</param>
    /// <param name="body">Data to post</param>
    /// <returns>Response body from server</returns>
    public string Post(string url, string body)
    {
      return HandleResponse(SendRequest(url, "POST", body));
    }

    /// <summary>
    /// Http PUT
    /// </summary>
    /// <param name="url">Where to put</param>
    /// <param name="body">Data to put</param>
    /// <returns></returns>
    public string Put(string url, string body)
    {
      return HandleResponse(SendRequest(url, "PUT", body));      
    }

    /// <summary>
    /// Http DELETE
    /// </summary>
    /// <param name="url">What to delete</param>
    /// <returns>Response body from server</returns>
    public string Delete(string url)
    {
      return HandleResponse(SendRequest(url, "DELETE", null));      
    }
  }
}