using System;
using System.Collections.Generic;
using Neo4RestNet.Core;
using Newtonsoft.Json.Linq;

namespace Neo4RestNet.Rest.Traverser
{
  public class ReturnableEvaluatorBuilder
  {
    private enum TypeOfReturnableEvaluator
    {
      BuiltIn,
      Javascript,
    }

    private readonly ReturnableEvaluator _returnableEvaluator;
    private readonly TypeOfReturnableEvaluator _typeOfReturnableEvaluator;
    private readonly string _returnableEvaluatorJs;

    private static readonly IDictionary<ReturnableEvaluator, string> _returnableEvaluatorNames
      = new Dictionary<ReturnableEvaluator, string>
          {
            { ReturnableEvaluator.All, "all" },
            { ReturnableEvaluator.AllButStartNode, "all but start" },
          };

    public ReturnableEvaluatorBuilder(ReturnableEvaluator returnableEvaluator)
    {
      _returnableEvaluator = returnableEvaluator;
      _typeOfReturnableEvaluator = TypeOfReturnableEvaluator.BuiltIn;
    }

    public ReturnableEvaluatorBuilder(string jsEvaluator)
    {
      _returnableEvaluatorJs = jsEvaluator;
      _typeOfReturnableEvaluator = TypeOfReturnableEvaluator.Javascript;
    }

    public string Build()
    {
      JObject expressionJson;

      switch (_typeOfReturnableEvaluator)
      {
        case TypeOfReturnableEvaluator.BuiltIn:
          expressionJson = BuildBuiltIn();
          break;
        case TypeOfReturnableEvaluator.Javascript:
          expressionJson = BuildJavascript();
          break;
        default:
          throw new ArgumentOutOfRangeException();
      }

      return new JProperty("return filter", expressionJson).ToString();
    }

    private JObject BuildJavascript()
    {
      var json = new JObject      
                   {
                     {"language", JToken.FromObject("javascript")},
                     {"body", JToken.FromObject(_returnableEvaluatorJs)}
                   };
      return json;
    }

    private JObject BuildBuiltIn()
    {
      var json = new JObject
                   {
                     {"language", JToken.FromObject("builtin")},
                     {"name", JToken.FromObject(_returnableEvaluatorNames[_returnableEvaluator])}
                   };
      return json;
    }
  }
}