﻿using Neo4RestNet.Core;
using Neo4RestNet.ObjectMapping;
using Neo4RestNet.ObjectMapping.Reflection;
using NUnit.Framework;

namespace Neo4RestNet.Tests
{
  public class TestClass
  {
    public int IntProperty { get; set; }
    public string StringProperty { get; set; }
    public object ObjProperty { get; set; }
  }

  [TestFixture]
  public class MapTests
  {
    private PropertyMap _propertyMap;

    [SetUp]
    public void Setup()
    {
      _propertyMap = new PropertyMap(new EmitReflector());
    }

    // ReSharper disable InconsistentNaming
    [Test]
    public void map_properties_from_obj_to_container()
    {
      var container = new PropertyContainer();
      var obj = new TestClass {IntProperty = 555, StringProperty = "s", ObjProperty = new object()};

      _propertyMap.ObjectToContainer(obj, container);

      Assert.That(container.GetProperty<int>("IntProperty"), Is.EqualTo(555));
      Assert.That(container.GetProperty<string>("StringProperty"), Is.EqualTo("s"));
      Assert.That(container.HasProperty("ObjProperty"), Is.False);
    }

    [Test]
    public void map_properties_from_container_to_obj()
    {
      var container = new PropertyContainer();
      var obj = new TestClass();
      container.SetProperty("IntProperty", 10);
      container.SetProperty("StringProperty", "uh");

      _propertyMap.Container2Object(container, obj);
      
      Assert.That(obj.IntProperty, Is.EqualTo(10));
      Assert.That(obj.StringProperty, Is.EqualTo("uh"));
    }

    // ReSharper restore InconsistentNaming
  }
}
