﻿using System.Linq;
using Neo4RestNet.Core;
using Neo4RestNet.Core.Extensions;
using Neo4RestNet.Rest;
using NUnit.Framework;

namespace Neo4RestNet.Tests
{
  [TestFixture]
  public class NodeTraversalExtensionTests
  {
    private IGraphDatabaseService _graphDb;

    [SetUp]
    public void Setup()
    {
      _graphDb = new RestGraphDatabaseService(Constants.DatabaseUrl);
    }

    // ReSharper disable InconsistentNaming
    [Test]
    public void traverse_with_stop_evaluator_expression()
    {
      var startnode = _graphDb.CreateNode(Constants.TestNode);
      var nodeAtDepthOne = _graphDb.CreateNode(Constants.TestNode);
      var nodeAtDepthTwo = _graphDb.CreateNode(Constants.TestNode);
      var nodeAtDepthThree = _graphDb.CreateNode(Constants.TestNode);
      startnode.CreateRelationshipTo(nodeAtDepthOne, "test");
      nodeAtDepthOne.CreateRelationshipTo(nodeAtDepthTwo, "test");
      nodeAtDepthTwo.CreateRelationshipTo(nodeAtDepthThree, "test");
      nodeAtDepthTwo.SetProperty("stop", true);

      var nodes = startnode.TraverseNodes(TraversalOrder.DepthFirst,
                                          p => p.Node.HasProperty("stop") && p.Node.GetProperty<bool>("stop"),
                                          ReturnableEvaluator.All, "test", Direction.All);

      Assert.That(nodes.Count(), Is.EqualTo(2));
      Assert.That(nodes.Where(n => n.Id == nodeAtDepthOne.Id).Count(), Is.EqualTo(1));
      Assert.That(nodes.Where(n => n.Id == nodeAtDepthTwo.Id).Count(), Is.EqualTo(1));
    }

    [Test]
    public void traverse_with_returnable_evaluator_expression()
    {
      var startnode = _graphDb.CreateNode(Constants.TestNode);
      var nodeAtDepthOne = _graphDb.CreateNode(Constants.TestNode);
      var nodeAtDepthTwo = _graphDb.CreateNode(Constants.TestNode);
      var nodeAtDepthThree = _graphDb.CreateNode(Constants.TestNode);
      startnode.CreateRelationshipTo(nodeAtDepthOne, "test");
      nodeAtDepthOne.CreateRelationshipTo(nodeAtDepthTwo, "test");
      nodeAtDepthTwo.CreateRelationshipTo(nodeAtDepthThree, "test");
      startnode.SetProperty("pickme", 1);
      nodeAtDepthThree.SetProperty("pickme", 1);

      var nodes = startnode.TraverseNodes(TraversalOrder.DepthFirst, StopEvaluator.EndOfGraph,
                                          p => p.Node.HasProperty("pickme"), "test", Direction.All);
//                                          "position.node().hasProperty('pickme')", "test", Direction.All);

      Assert.That(nodes.Count(), Is.EqualTo(2));
      Assert.That(nodes.Where(n => n.Id == startnode.Id).Count(), Is.EqualTo(1));
      Assert.That(nodes.Where(n => n.Id == nodeAtDepthThree.Id).Count(), Is.EqualTo(1));
    }
    // ReSharper restore InconsistentNaming

  }
}
