﻿using System.Collections.Generic;
using Neo4RestNet.Core;
using NUnit.Framework;

namespace Neo4RestNet.Tests
{
  [TestFixture]
  public class PropertyContainerTests
  {
    private PropertyContainer _propertyContainer;

    [SetUp]
    public void Setup()
    {
      _propertyContainer = new PropertyContainer(new Dictionary<string, object>
                      {
                        {"key1", "val1"},
                        {"key2", "val2"}
                      });
    }

    // ReSharper disable InconsistentNaming
    [Test]
    public void get_existing_property_should_succeed()
    {
      var val = _propertyContainer.GetProperty<string>("key1");
      Assert.That(val, Is.EqualTo("val1"));
    }

    [Test]
    public void get_non_existing_property_should_throw()
    {
      Assert.Throws<NotFoundException>(() => _propertyContainer.GetProperty<string>("non existing"));
    }

    [Test]
    public void get_non_existing_property_with_default_should_return_default()
    {
      var val = _propertyContainer.GetProperty<string>("non existing", "default value");
      Assert.That(val, Is.EqualTo("default value"));
    }

    // ReSharper restore InconsistentNaming
  }
}
