﻿using Neo4RestNet.Core;
using Neo4RestNet.Rest;
using NUnit.Framework;

namespace Neo4RestNet.Tests
{
  // Todo: Keep track of all nodes created and clean up after us when the tests are done

  [TestFixture]
  public class RestGraphDbTests
  {
    private IGraphDatabaseService _graphDb;
    private const string InvalidNode = "{\"test\": \"true\"";

    [SetUp]
    public void Setup()
    {
      _graphDb = new RestGraphDatabaseService(Constants.DatabaseUrl);
    }

    // ReSharper disable InconsistentNaming
    [Test]
    public void create_node_should_succeed()
    {
      var node = _graphDb.CreateNode(Constants.TestNode);

      Assert.That(node, Is.Not.Null);
      Assert.That(node.HasProperty("test"), Is.True);
    }

    [Test]
    public void create_empty_node_should_succeed()
    {
      var node = _graphDb.CreateNode();

      Assert.That(node, Is.Not.Null);
    }

    [Test]
    public void create_invalid_node_should_throw()
    {
      Assert.Throws<GraphDbException>(() => _graphDb.CreateNode(InvalidNode));
    }

    [Test]
    public void get_referencenode_should_succeed()
    {
      var node = _graphDb.GetReferenceNode();

      Assert.That(node, Is.Not.Null);
    }

    [Test]
    public void get_node_by_id_should_succeed()
    {
      // Retrieve reference node and try getting it by id
      var referenceNode = _graphDb.GetReferenceNode();
      var node = _graphDb.GetNode(referenceNode.Id);

      Assert.That(node, Is.Not.Null);
    }

    [Test]
    public void get_non_existing_node_by_id_should_throw()
    {
      var node = _graphDb.GetNode(-1);

      Assert.Throws<GraphDbException>(() => node.HasProperty("test"));
    }

    // ReSharper restore InconsistentNaming
  }
}
