﻿using System.Linq;
using Neo4RestNet.Core;
using Neo4RestNet.Rest;
using NUnit.Framework;

namespace Neo4RestNet.Tests
{
  [TestFixture]
  public class RestRelationshipTests
  {
    private RestGraphDatabaseService _graphDb;

    [SetUp]
    public void Setup()
    {
      _graphDb = new RestGraphDatabaseService(Constants.DatabaseUrl);
    }


    // ReSharper disable InconsistentNaming
    [Test]
    public void setting_a_property_on_a_new_relationship_should_set()
    {
      var node1 = _graphDb.CreateNode(Constants.TestNode);
      var node2 = _graphDb.CreateNode(Constants.TestNode);
      var relationship = node1.CreateRelationshipTo(node2, "test");

      relationship.SetProperty("newProperty", 1);
      var relationships = node1.GetRelationships(Direction.Outgoing);

      relationship = relationships.First();

      Assert.That(relationship.GetProperty<int>("newProperty"), Is.EqualTo(1));
    }

    [Test]
    public void deleting_a_relationship_should_delete()
    {
      var node1 = _graphDb.CreateNode(Constants.TestNode);
      var node2 = _graphDb.CreateNode(Constants.TestNode);
      var relationship = node1.CreateRelationshipTo(node2, "test");

      relationship.Delete();

      var relationships = node1.GetRelationships(Direction.Outgoing);
      
      Assert.That(relationships.Count(), Is.EqualTo(0));
    }

    [Test]
    public void other_node_should_return_other_node_in_relation()
    {
      var node1 = _graphDb.CreateNode(Constants.TestNode);
      var node2 = _graphDb.CreateNode(Constants.TestNode);
      var relationship = node1.CreateRelationshipTo(node2, "test");

      Assert.That(relationship.GetOtherNode(node1).Id, Is.EqualTo(node2.Id));
      Assert.That(relationship.GetOtherNode(node2).Id, Is.EqualTo(node1.Id));
    }

    [Test]
    public void when_node_not_part_of_relationship_othernode_should_throw()
    {
      var node1 = _graphDb.CreateNode(Constants.TestNode);
      var node2 = _graphDb.CreateNode(Constants.TestNode);
      var nodex = _graphDb.CreateNode(Constants.TestNode);
      var relationship = node1.CreateRelationshipTo(node2, "test");

      Assert.Throws<GraphDbException>(() => relationship.GetOtherNode(nodex));
    }

    // ReSharper restore InconsistentNaming
  }
}
